
/* lots of functions for summing the first n integers */

#include <stdio.h>
#include <stdlib.h> /* for atoi */

// point: you do not need a loop or recursion for this math problem
int sum1(int n) { 
  return ((n+1)*n) / 2;
}

// point: if/then/else and recursion like in Java, but recursion
//        does "take up stack space" -- n frames worsh
int sum2(int n) {
  if(n==0) {
    return 0;
  } else {
    return sum2(n-1) + n;
  }
}

// point: just like sum2 but without "redundant" braces and else
int sum3(int n) {
  if(n==0)
    return 0;
  return sum3(n-1) + n;
}

// point: a while loop is more efficient here, +=, -=, local vars like in Java
int sum4(int n) {
  int ans = 0;
  while(n != 0) {
    ans += n;
    n -= 1;
  }
  return ans;
}

// point: Tests in C work on integers so "n != 0" is a lot like n
//        (still n != 0 probably better style; people disagree)
int sum5(int n) {
  int ans = 0;
  while(n) {
    ans += n;
    --n; /* or n-- or n = n - 1 */
  }
  return ans;
}

// point: n-- and --n are different when the result is used as part
//        of a larger expression
int sum6(int n) {
  int ans = 0;
  while(n)
    ans += n--; /* NOT --n */
  return ans;
}

// point: for loops often more concise and easier to read
int sum7(int n) {
  int ans = 0;
  for(; n >= 0; --n) {
    ans += n;
  }
  return ans;
}

// point: for loops can be abused to be confusing to read
//        here we have two "next" expressions and no body
int sum8(int n) {
  int ans = 0;
  for(; n >= 0; ans+=n, --n) 
    ;
  return ans;
}

// point: Can use break (poor style here)
int sum9(int n) {
  int ans = 0;
  while(1) {
    if(n==0)
      break;
    ans += n--;
  }
  return ans;
}

// point: a return can also end an otherwise-infinite-loop
int sum10(int n) {
  int ans = 0;
  while(1) {
    if(n==0)
      return ans;
    ans += n--;
  }
}
    
// point: C has labels and goto, usually bad style but it has its uses
//        (This is not a good use; it's a purposely unreadable one)
int sum11(int n) {
  int ans = 0;
  goto L2;
 L1: return ans;
 L2:
  if(n==0)
    goto L1;
  ans += n--;
  goto L2;
}

// point: learn about switch, beware implicit fall-through
//        (Using switch here is not good style and the whole
//         algorithm here is unnessarily confusing.)
int sum12(int n) {
  int ans = 0;
  switch(n) {
  case 0: return 0;
  case 3: return 6;
  case 4: ans = 7; // fall-through!
  case 2: ans += 3; break;
  default: 
    for(; n >= 0; --n)
      ans += n;
  }
  return ans;
}

// point: This code is illegal because you goto into the scope of ans.
int sumWrong(int n) {
  if(n==0)
    return 0;
  goto L2;
  {int ans = 0;
 L1: return ans;
 L2:
  if(n==0)
    goto L1;
  ans += n--;
  goto L2;
  }
}

// point: use a main and fprintf to check command-line arguments and test
//        the functions above
int main(int argc, char**argv) {
  if(argc != 2) {
    fprintf(stderr,"bad args\n");
    return 1;
  }
  { int num = atoi(argv[1]);
  /* note: see strtol for better error checking */
  if(num < 0) {
    fprintf(stderr,"bad args\n");
    return 1;
  }
  /* later we will use function pointers to avoid this copy/paste */
  fprintf(stdout, "1: %d\n", sum1(num));
  fprintf(stdout, "2: %d\n", sum2(num));
  fprintf(stdout, "3: %d\n", sum3(num));
  fprintf(stdout, "4: %d\n", sum4(num));
  fprintf(stdout, "5: %d\n", sum5(num));
  fprintf(stdout, "6: %d\n", sum6(num));
  fprintf(stdout, "7: %d\n", sum7(num));
  fprintf(stdout, "8: %d\n", sum8(num));
  fprintf(stdout, "9: %d\n", sum9(num));
  fprintf(stdout, "10: %d\n", sum10(num));
  fprintf(stdout, "11: %d\n", sum11(num));
  fprintf(stdout, "12: %d\n", sum12(num));
  fprintf(stdout, "wrong: %d\n", sumWrong(num));
  }
  return 0;
}
